@@ -9,13 +9,13 @@ from email.mime.text import MIMEText
 from email.header import Header
 from ast import literal_eval
 
-from ..model import ModelView, ModelSQL, fields
-from ..tools import get_smtp_server
+from ..model import ModelView, ModelSQL, fields, dualmethod
 from ..transaction import Transaction
 from ..pool import Pool
 from .. import backend
 from ..config import config
 from ..cache import Cache
+from ..sendmail import sendmail
 
 __all__ = [
     'Cron',
@@ -70,15 +70,20 @@ class Cron(ModelSQL, ModelView):
                 'request_body': ("The following action failed to execute "
                     "properly: \"%s\"\n%s\n Traceback: \n\n%s\n")
                 })
+        cls._buttons.update({
+                'run_once': {
+                    'icon': 'tryton-executable',
+                    },
+                })
 
     @classmethod
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
-        cursor = Transaction().cursor
+        cursor = Transaction().connection.cursor()
         cron = cls.__table__()
 
         # Migration from 2.0: rename numbercall, doall and nextcall
-        table = TableHandler(cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
         table.column_rename('numbercall', 'number_calls')
         table.column_rename('doall', 'repeat_missed')
         table.column_rename('nextcall', 'next_call')
@@ -126,59 +131,56 @@ class Cron(ModelSQL, ModelView):
         '''
         return _INTERVALTYPES[cron.interval_type](cron.interval_number)
 
-    @classmethod
-    def send_error_message(cls, cron):
-        tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
-        tb_s = tb_s.decode('utf-8', 'ignore')
-        subject = cls.raise_user_error('request_title',
-            raise_exception=False)
-        body = cls.raise_user_error('request_body',
-            (cron.name, cron.__url__, tb_s),
-            raise_exception=False)
-
-        from_addr = config.get('email', 'from')
-        to_addr = cron.request_user.email
-
-        msg = MIMEText(body, _charset='utf-8')
-        msg['To'] = to_addr
-        msg['From'] = from_addr
-        msg['Subject'] = Header(subject, 'utf-8')
-        if not to_addr:
-            logger.error(msg.as_string())
-        else:
-            try:
-                server = get_smtp_server()
-                server.sendmail(from_addr, to_addr, msg.as_string())
-                server.quit()
-            except Exception:
-                logger.error('Unable to deliver email:\n %s',
-                    msg.as_string(), exc_info=True)
-
-    @classmethod
-    def _callback(cls, cron):
+    def send_error_message(self):
         pool = Pool()
         Config = pool.get('ir.configuration')
-        try:
-            args = (cron.args or []) and literal_eval(cron.args)
+
+        if self.request_user.language:
+            language = self.request_user.language.code
+        else:
+            language = Config.get_language()
+
+        with Transaction().set_user(self.user.id), \
+                Transaction().set_context(language=language):
+            tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
+            tb_s = tb_s.decode('utf-8', 'ignore')
+            subject = self.raise_user_error('request_title',
+                raise_exception=False)
+            body = self.raise_user_error('request_body',
+                (self.name, self.__url__, tb_s),
+                raise_exception=False)
+
+            from_addr = config.get('email', 'from')
+            to_addr = self.request_user.email
+
+            msg = MIMEText(body, _charset='utf-8')
+            msg['To'] = to_addr
+            msg['From'] = from_addr
+            msg['Subject'] = Header(subject, 'utf-8')
+            if not to_addr:
+                logger.error(msg.as_string())
+            else:
+                sendmail(from_addr, to_addr, msg)
+
+    @dualmethod
+    @ModelView.button
+    def run_once(cls, crons):
+        pool = Pool()
+        for cron in crons:
+            if cron.args:
+                args = literal_eval(cron.args)
+            else:
+                args = []
             Model = pool.get(cron.model)
             with Transaction().set_user(cron.user.id):
                 getattr(Model, cron.function)(*args)
-        except Exception:
-            Transaction().cursor.rollback()
-
-            req_user = cron.request_user
-            language = (req_user.language.code if req_user.language
-                    else Config.get_language())
-            with Transaction().set_user(cron.user.id), \
-                    Transaction().set_context(language=language):
-                cls.send_error_message(cron)
 
     @classmethod
     def run(cls, db_name):
         now = datetime.datetime.now()
         with Transaction().start(db_name, 0) as transaction:
             Cache.clean(db_name)
-            transaction.cursor.lock(cls._table)
+            transaction.database.lock(transaction.connection, cls._table)
             crons = cls.search([
                     ('number_calls', '!=', 0),
                     ('next_call', '<=', datetime.datetime.now()),
@@ -191,7 +193,11 @@ class Cron(ModelSQL, ModelView):
                     first = True
                     while next_call < now and number_calls != 0:
                         if first or cron.repeat_missed:
-                            cls._callback(cron)
+                            try:
+                                cron.run_once()
+                            except Exception:
+                                transaction.rollback()
+                                cron.send_error_message()
                         next_call += cls.get_delta(cron)
                         if number_calls > 0:
                             number_calls -= 1
@@ -202,8 +208,8 @@ class Cron(ModelSQL, ModelView):
                     if not number_calls:
                         cron.active = False
                     cron.save()
-                    transaction.cursor.commit()
+                    transaction.commit()
                 except Exception:
-                    transaction.cursor.rollback()
+                    transaction.rollback()
                     logger.error('Running cron %s', cron.id, exc_info=True)
             Cache.resets(db_name)
