@@ -10,7 +10,6 @@ import datetime
 from trytond.model import ModelSQL, fields
 from trytond.config import config
 from .. import backend
-from ..transaction import Transaction
 
 __all__ = [
     'Session', 'SessionWizard',
@@ -34,7 +33,7 @@ class Session(ModelSQL):
         TableHandler = backend.get('TableHandler')
         super(Session, cls).__register__(module_name)
 
-        table = TableHandler(Transaction().cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
         table.index_action('create_uid', 'add')
 
     @staticmethod
