@@ -193,7 +193,8 @@ class View(ModelSQL, ModelView):
         if self.name and self.module:
             path = os.path.join(self.module, 'view', self.name + '.xml')
             try:
-                with file_open(path, subdir='modules', mode='r') as fp:
+                with file_open(path,
+                        subdir='modules', mode='r', encoding='utf-8') as fp:
                     value = fp.read()
             except IOError:
                 pass
