@@ -5,10 +5,6 @@ import datetime
 import time
 import csv
 import warnings
-try:
-    import cStringIO as StringIO
-except ImportError:
-    import StringIO
 
 from decimal import Decimal
 from itertools import islice, ifilter, chain, izip
@@ -25,7 +21,7 @@ from trytond.const import OPERATORS
 from trytond.config import config
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond.cache import LRUDict, freeze
+from trytond.cache import LRUDict, LRUDictTransaction, freeze
 from trytond import backend
 from trytond.rpc import RPC
 from .modelview import ModelView
@@ -170,8 +166,8 @@ class ModelStorage(Model):
             if local_cache:
                 local_cache.clear()
 
-        # Clean cursor cache
-        for cache in Transaction().cursor.cache.itervalues():
+        # Clean transaction cache
+        for cache in Transaction().cache.itervalues():
             if cls.__name__ in cache:
                 for record in all_records:
                     if record.id in cache[cls.__name__]:
@@ -224,8 +220,8 @@ class ModelStorage(Model):
         # Increase transaction counter
         Transaction().counter += 1
 
-        # Clean cursor cache
-        for cache in Transaction().cursor.cache.values():
+        # Clean transaction cache
+        for cache in Transaction().cache.values():
             for cache in (cache, cache.get('_language_cache', {}).values()):
                 if cls.__name__ in cache:
                     for record in records:
@@ -454,9 +450,14 @@ class ModelStorage(Model):
         '''
         Return a list of instance for the ids
         '''
+        transaction = Transaction()
         ids = map(int, ids)
-        local_cache = LRUDict(cache_size())
-        return [cls(int(x), _ids=ids, _local_cache=local_cache) for x in ids]
+        local_cache = LRUDictTransaction(cache_size())
+        transaction_cache = transaction.get_cache()
+        return [cls(x, _ids=ids,
+                _local_cache=local_cache,
+                _transaction_cache=transaction_cache,
+                _transaction=transaction) for x in ids]
 
     @staticmethod
     def __export_row(record, fields_names):
@@ -569,7 +570,7 @@ class ModelStorage(Model):
                 return None
             res = []
             Relation = pool.get(relation)
-            for word in csv.reader(StringIO.StringIO(value), delimiter=',',
+            for word in csv.reader(value.splitlines(), delimiter=',',
                     quoting=csv.QUOTE_NONE, escapechar='\\').next():
                 res2 = Relation.search([
                     ('rec_name', '=', word),
@@ -618,7 +619,7 @@ class ModelStorage(Model):
             relation = None
             ftype = fields_def[field[-1][:-3]]['type']
             if ftype == 'many2many':
-                value = csv.reader(StringIO.StringIO(value), delimiter=',',
+                value = csv.reader(value.splitlines(), delimiter=',',
                         quoting=csv.QUOTE_NONE, escapechar='\\').next()
             elif ftype == 'reference':
                 try:
@@ -785,7 +786,7 @@ class ModelStorage(Model):
             if values is None:
                 return False
             for model_data in models_data:
-                if not model_data.values:
+                if not model_data.values or model_data.noupdate:
                     continue
                 xml_values = ModelData.load_values(model_data.values)
                 for key, val in values.iteritems():
@@ -1136,6 +1137,8 @@ class ModelStorage(Model):
         pool = Pool()
         vals = {}
         for field in defaults.keys():
+            if '.' in field:  # skip all related fields
+                continue
             fld_def = cls._fields[field]
             if fld_def._type in ('many2one', 'one2one'):
                 if isinstance(defaults[field], (list, tuple)):
@@ -1159,9 +1162,13 @@ class ModelStorage(Model):
     def __init__(self, id=None, **kwargs):
         _ids = kwargs.pop('_ids', None)
         _local_cache = kwargs.pop('_local_cache', None)
-        self._cursor = Transaction().cursor
-        self._user = Transaction().user
-        self._context = Transaction().context
+        _transaction_cache = kwargs.pop('_transaction_cache', None)
+        transaction = kwargs.pop('_transaction', None)
+        if transaction is None:
+            transaction = Transaction()
+        self._transaction = transaction
+        self._user = transaction.user
+        self._context = transaction.context
         if id is not None:
             id = int(id)
         if _ids is not None:
@@ -1170,19 +1177,22 @@ class ModelStorage(Model):
         else:
             self._ids = [id]
 
-        self._cursor_cache = self._cursor.get_cache()
+        if _transaction_cache is not None:
+            self._transaction_cache = _transaction_cache
+        else:
+            self._transaction_cache = transaction.get_cache()
 
         if _local_cache is not None:
+            assert isinstance(_local_cache, LRUDictTransaction)
             self._local_cache = _local_cache
         else:
-            self._local_cache = LRUDict(cache_size())
-        self._local_cache.counter = Transaction().counter
+            self._local_cache = LRUDictTransaction(cache_size())
 
         super(ModelStorage, self).__init__(id, **kwargs)
 
     @property
     def _cache(self):
-        cache = self._cursor_cache
+        cache = self._transaction_cache
         if self.__name__ not in cache:
             cache[self.__name__] = LRUDict(cache_size())
         return cache[self.__name__]
@@ -1191,13 +1201,10 @@ class ModelStorage(Model):
         try:
             return super(ModelStorage, self).__getattr__(name)
         except AttributeError:
-            if self.id < 0:
+            if self.id is None or self.id < 0:
                 raise
 
-        counter = Transaction().counter
-        if self._local_cache.counter != counter:
-            self._local_cache.clear()
-            self._local_cache.counter = counter
+        self._local_cache.refresh()
 
         # fetch the definition of the field
         try:
@@ -1279,7 +1286,8 @@ class ModelStorage(Model):
         index = self._ids.index(self.id)
         ids = chain(islice(self._ids, index, None),
             islice(self._ids, 0, max(index - 1, 0)))
-        ids = islice(unique(ifilter(filter_, ids)), self._cursor.IN_MAX)
+        ids = islice(unique(ifilter(filter_, ids)),
+            self._transaction.database.IN_MAX)
 
         def instantiate(field, value, data):
             if field._type in ('many2one', 'one2one', 'reference'):
@@ -1300,6 +1308,7 @@ class ModelStorage(Model):
                     Model = field.get_target()
             except KeyError:
                 return value
+            transaction = Transaction()
             ctx = {}
             if field.context:
                 pyson_context = PYSONEncoder().encode(field.context)
@@ -1308,30 +1317,33 @@ class ModelStorage(Model):
             if getattr(field, 'datetime_field', None):
                 datetime_ = data.get(field.datetime_field)
                 ctx = {'_datetime': datetime_}
-            with Transaction().set_context(**ctx):
+            with transaction.set_context(**ctx):
+                kwargs = {}
                 key = (Model, freeze(ctx))
-                local_cache = model2cache.setdefault(key,
-                    LRUDict(cache_size()))
-                ids = model2ids.setdefault(key, [])
+                kwargs['_local_cache'] = model2cache.setdefault(key,
+                    LRUDictTransaction(cache_size()))
+                kwargs['_ids'] = ids = model2ids.setdefault(key, [])
+                kwargs['_transaction_cache'] = transaction.get_cache()
+                kwargs['_transaction'] = transaction
                 if field._type in ('many2one', 'one2one', 'reference'):
+                    value = int(value)
                     ids.append(value)
-                    return Model(value, _ids=ids, _local_cache=local_cache)
+                    return Model(value, **kwargs)
                 elif field._type in ('one2many', 'many2many'):
-                    ids.extend(value)
-                    return tuple(Model(id, _ids=ids, _local_cache=local_cache)
-                        for id in value)
+                    ids.extend(int(x) for x in value)
+                    return tuple(Model(id, **kwargs) for id in value)
 
         model2ids = {}
         model2cache = {}
         # Read the data
-        with Transaction().set_cursor(self._cursor), \
-                Transaction().set_user(self._user), \
-                Transaction().set_context(self._context):
+        with Transaction().set_current_transaction(self._transaction), \
+                self._transaction.set_user(self._user), \
+                self._transaction.set_context(self._context):
             if self.id in self._cache and name in self._cache[self.id]:
                 # Use values from cache
                 ids = islice(chain(islice(self._ids, index, None),
                         islice(self._ids, 0, max(index - 1, 0))),
-                    self._cursor.IN_MAX)
+                    self._transaction.database.IN_MAX)
                 ffields = {name: ffields[name]}
                 read_data = [{'id': i, name: self._cache[i][name]}
                     for i in ids
@@ -1373,7 +1385,8 @@ class ModelStorage(Model):
             field = self._fields[fname]
             if field._type in ('many2one', 'one2one', 'reference'):
                 if value:
-                    if value.id < 0 and field._type != 'reference':
+                    if ((value.id is None or value.id < 0)
+                            and field._type != 'reference'):
                         value.save()
                     if field._type == 'reference':
                         value = str(value)
@@ -1381,7 +1394,7 @@ class ModelStorage(Model):
                         value = value.id
             if field._type in ('one2many', 'many2many'):
                 targets = value
-                if self.id >= 0:
+                if self.id is not None and self.id >= 0:
                     _values, self._values = self._values, None
                     try:
                         to_remove = [t.id for t in getattr(self, fname)]
@@ -1393,7 +1406,7 @@ class ModelStorage(Model):
                 to_create = []
                 to_write = []
                 for target in targets:
-                    if target.id < 0:
+                    if target.id is None or target.id < 0:
                         if field._type == 'one2many':
                             # Don't store old target link
                             setattr(target, field.field, None)
@@ -1427,11 +1440,11 @@ class ModelStorage(Model):
         save_values = {}
         to_create = []
         to_write = []
-        cursor = records[0]._cursor
+        transaction = records[0]._transaction
         user = records[0]._user
         context = records[0]._context
         for record in records:
-            assert cursor == record._cursor
+            assert transaction == record._transaction
             assert user == record._user
             assert context == record._context
             save_values[record] = record._save_values
@@ -1443,14 +1456,14 @@ class ModelStorage(Model):
                 to_write.append(record)
         transaction = Transaction()
         try:
-            with transaction.set_cursor(cursor), \
+            with transaction.set_current_transaction(transaction), \
                     transaction.set_user(user), \
                     transaction.set_context(context):
                 if to_create:
                     news = cls.create([save_values[r] for r in to_create])
                     for record, new in izip(to_create, news):
                         record._ids.remove(record.id)
-                        record.id = new.id
+                        record._id = new.id
                         record._ids.append(record.id)
                 if to_write:
                     cls.write(*sum(
