@@ -8,7 +8,7 @@ except ImportError:
 from trytond.pool import Pool
 from trytond.config import config
 from trytond.transaction import Transaction
-from trytond.exceptions import NotLogged
+from trytond import backend
 
 
 def _get_pool(dbname):
@@ -20,24 +20,22 @@ def _get_pool(dbname):
 
 
 def login(dbname, loginname, password, cache=True):
-    with Transaction().start(dbname, 0) as transaction:
+    with Transaction().start(dbname, 0):
         pool = _get_pool(dbname)
         User = pool.get('res.user')
         user_id = User.get_login(loginname, password)
-        transaction.cursor.commit()
     if user_id:
         if not cache:
             return user_id
-        with Transaction().start(dbname, user_id) as transaction:
+        with Transaction().start(dbname, user_id):
             Session = pool.get('ir.session')
             session, = Session.create([{}])
-            transaction.cursor.commit()
             return user_id, session.key
-    return False
+    return
 
 
 def logout(dbname, user, session):
-    with Transaction().start(dbname, 0) as transaction:
+    with Transaction().start(dbname, 0):
         pool = _get_pool(dbname)
         Session = pool.get('ir.session')
         sessions = Session.search([
@@ -48,7 +46,6 @@ def logout(dbname, user, session):
         session, = sessions
         name = session.create_uid.login
         Session.delete(sessions)
-        transaction.cursor.commit()
     return name
 
 
@@ -60,17 +57,19 @@ def check_super(passwd):
 
 
 def check(dbname, user, session):
-    if user == 0:
-        raise Exception('AccessDenied')
-    if not user:
-        raise NotLogged()
-    with Transaction().start(dbname, user) as transaction:
-        pool = _get_pool(dbname)
-        Session = pool.get('ir.session')
-        try:
-            if not Session.check(user, session):
-                raise NotLogged()
-            else:
-                return user
-        finally:
-            transaction.cursor.commit()
+    DatabaseOperationalError = backend.get('DatabaseOperationalError')
+    for count in range(config.getint('database', 'retry'), -1, -1):
+        with Transaction().start(dbname, user) as transaction:
+            pool = _get_pool(dbname)
+            Session = pool.get('ir.session')
+            try:
+                if not Session.check(user, session):
+                    return
+                else:
+                    return user
+            except DatabaseOperationalError:
+                if count:
+                    continue
+                raise
+            finally:
+                transaction.commit()
